/*****************************************************************************
 *   Copyright (C) 2005 by HECHT Franck                                      *
 *   franck.hecht@planete-c.com                                              *
 *                                                                           *
 *   This program is free software; you can redistribute it and/or modify    *
 *   it under the terms of the GNU General Public License as published by    *
 *   the Free Software Foundation; either version 2 of the License, or       *
 *   (at your option) any later version.                                     *
 *                                                                           *
 *   This program is distributed in the hope that it will be useful,         *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *   GNU General Public License for more details.                            *
 *                                                                           *
 *   You should have received a copy of the GNU General Public License       *
 *   along with this program; if not, write to the                           *
 *   Free Software Foundation, Inc.,                                         *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.               *
 *****************************************************************************/
 
/* --------------------------------------------------------------------------
   Module   :  msg_dlg
   Fichier  :  msg_dlg.c
   
   Creation :  05/07/2005 - 20:31
   
   But      :  Affiche une fenetre de message avec GTK.
   --------------------------------------------------------------------------
   Dernières modifications:

   11/02/2006 - 14:51:  Ajout de la prise en charge des message en UTF8.
   -------------------------------------------------------------------------- */
 
 
#include "msg_dlg.h"


/* macros =================================================================== */
/* constants ================================================================ */

/* Taille maximum d'un message dans une boite de message. */
#define MAX_LEN_MSG 256


/* types ==================================================================== */
/* structures =============================================================== */
/* private variables ======================================================== */
/* private functions ======================================================== */
/* internal public functions ================================================ */

/* --------------------------------------------------------------------------
   show_message
   --------------------------------------------------------------------------
   Fonction affichant une boite de message en GTK.
   --------------------------------------------------------------------------
   E: Fenetre parent.
   E: Constantes predefinies de GTK affichant le style de l'icone:
         - GTK_MESSAGE_INFO,
         - GTK_MESSAGE_WARNING,
         - GTK_MESSAGE_QUESTION,
         - GTK_MESSAGE_ERROR
                           
   E: Constantes predefinies de GTK pour les boutons que doit contenir
      la boite de message:
         - GTK_BUTTONS_NONE,
         - GTK_BUTTONS_OK,
         - GTK_BUTTONS_CLOSE,
         - GTK_BUTTONS_CANCEL,
         - GTK_BUTTONS_YES_NO,
         - GTK_BUTTONS_OK_CANCEL
                           
   E: Chaine de caractere formant le message a afficher.
   E: Liste variable d'argument, utilise dans notre cas pour inserer des
      donnees dans la chaine 'message' comme on le ferais pour la
      fonction 'printf'.
      
   S: Valeur correspondants aux constantes de GTK suivantes:
         - GTK_RESPONSE_NONE
         - GTK_RESPONSE_REJECT
         - GTK_RESPONSE_ACCEPT
         - GTK_RESPONSE_DELETE_EVENT
         - GTK_RESPONSE_OK
         - GTK_RESPONSE_CANCEL
         - GTK_RESPONSE_CLOSE
         - GTK_RESPONSE_YES
         - GTK_RESPONSE_NO
         - GTK_RESPONSE_APPLY
         - GTK_RESPONSE_HELP
   -------------------------------------------------------------------------- */
gint show_message (GtkWindow *p_parent,
                   GtkMessageType message_type,
                   GtkButtonsType buttons_type,
                   const gchar *p_message,
                   ...)
gint show_message (GtkWindow *p_parent,
                   GtkMessageType message_type,
                   GtkButtonsType buttons_type,
                   const gchar *p_message,
                   ...)
{
   GtkWidget *p_dialog = NULL;
   gchar p_msg[MAX_LEN_MSG];
   gint ret_val;
   va_list params;
   gchar * s_utf8 = NULL;
   
   
   /* Recuperation de la liste variable. */
   va_start (params, p_message);

   
   /* Impression de la chaine finale. */
   vsprintf (p_msg, p_message, params);


   /* Transformation de la chaine au format UT8. */
   s_utf8 = g_locale_to_utf8 (p_msg, -1, NULL, NULL, NULL);
   
   
   /* Creation de la boite de message. */
   p_dialog = gtk_message_dialog_new (
                  p_parent,
                  GTK_DIALOG_MODAL,
                  message_type,
                  buttons_type,
                  s_utf8);


   /* Liberation de la chaine temporaire. */
   g_free (s_utf8);


   /* Lancement de la boite de message. */
   ret_val = gtk_dialog_run (GTK_DIALOG (p_dialog));
   
   
   /* Suppresion de la boite de message. */
   gtk_widget_destroy (p_dialog);
   
   
   /* Fin de la liste variable. */
   va_end (params);
   
   
   return ret_val;
}


/* public variables ========================================================= */
